//---------------------------------------------------------------------------------------------------------------------
// Copyright (c) AUMA Riester GmbH & Co. KG (2014). All Rights Reserved
// Target System: SIMATIC PDM
//---------------------------------------------------------------------------------------------------------------------
// Description:
// * Definitions for ServiceChannel tunneled over DPV1
//   - COMMANDs for read/write
//   - buffer variables
//
//---------------------------------------------------------------------------------------------------------------------
//  (c) Sipos Aktorik GmbH (Mathias Monse <mathias.monse@sipos.de>)
//---------------------------------------------------------------------------------------------------------------------

// ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// DEFINE
// ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#if defined (_V820_)        // from PDM v8.2.0 LISTS in COMMANDS are usable
#define  SCCOM_WITH_LISTS   // undefine it do use byte variables instead of LISTs
#endif

#define  SC_OKAY       0x0000
#define  SC_NOT_FOUND  0xFFFE
#define  SC_ERROR      0xFFFF


///*** DPV1 settings
#define DPV1_ACCESS_SLOT          253   // see Doors P056.D03-1.3.27.2
#define DPV1_FT_REQ_INDEX         10    // see Doors P056.D03-1.3.27.2 (Service commander request)
#define DPV1_FT_RSP_INDEX         11    // see Doors P056.D03-1.3.27.2 (Read from service commander)
#define DPV1_IDENT_INDEX          20    // see Doors P056.D03-1.3.27.2 (I&M block (97 Bytes))

///*** TIMEOUT settings
#define SC_TIMEOUT                1000 //90000 // ms
#define SC_ACCESS_WAIT            1    // s 
#define SC_ACCESS_TRIALS          (SC_TIMEOUT/(1000*SC_ACCESS_WAIT))
#define SC_READ_TIMEOUT           1000 // ms
#define SC_WRITE_TIMEOUT          1000 // ms
#define SC_REFRESH_MAX            10

///*** constant telegram entries
#define SC_IGNORE_BYTE              0x00
#define SC_SRC                      0x00
#define SC_DST                      0x01
#define SC_SRV_REQ                  0x11
#define SC_SRV_RSP                  0x22
#define SC_SRV_PND                  0x33
#define SC_SRV_REF                  0x44
#define SC_SRV_NOI                  0x55

#define SC_TELE_LEN_MAX             240   // max dpv1 telegram length 
#define SC_L4HEAD_LEN               5
#define SC_L4TAIL_LEN               2
#define SC_L4FRAME_LEN              (SC_L4HEAD_LEN + SC_L4TAIL_LEN)
#define SC_L5HEAD_LEN               2
#define FT_DATA_LEN               231   // (FT_TELE_LEN_MAX - SC_L4HEAD_LEN - SC_L5HEAD_LEN - SC_L4TAIL_LEN)

#define SC_LEN_FT_OBEX_HEADER        3
#define SC_LEN_FT_SETPATH_HEADER     (SC_LEN_FT_OBEX_HEADER+2)  // OBEX header with 2 flags bytes
#define SC_LEN_FT_CONNECT_HEADER     (SC_LEN_FT_OBEX_HEADER+4)  // OBEX header with 4 bytes (VER, FLG, MLEN)
#define SC_LEN_FT_CONN_ID_HEADER     5    // OBEX header id with 4 bytes CONN-ID (no length attribute)
#define SC_LEN_FT_FBGUID_HEADER     (SC_LEN_FT_OBEX_HEADER+16)  // OBEX header with 16 byte ASCII string
#define SC_LEN_FT_GET                5    // SC length of get request (excl 5 octets SC header + 2 octets checksum)
#define SC_LEN_FT_GET_HEADER         SC_LEN_FT_OBEX_HEADER

///*** Request Codes
#define FT_REQ_CONNECT              0x80
#define FT_REQ_DISCONNECT           0x81
#define FT_REQ_SETPATH              0x85
#define FT_REQ_GET                  0x03
#define FT_REQ_GET_F                0x83

///*** Response Codes
#define FT_RSP_OK                   0xA0
#define FT_RSP_CONTINUE             0x90
#define FT_RSP_NOT_FOUND            0xC4
#define FT_RSP_CHECK_ERROR          0xFC
#define FT_RSP_ACCESS_DENIED        0xFD
#define FT_RSP_INV_FUNCTION         0xFE
#define FT_RSP_WRONG_REQ_LENGTH     0xFF

///*** Telegram structure
#define SC_START_L4HEAD             0
#define SC_START_L5HEAD             (SC_START_L4HEAD + SC_L4HEAD_LEN)           // 5
#define SC_START_FTHEAD1            (SC_START_L5HEAD + SC_L5HEAD_LEN)           // 7
#define SC_START_FTHEAD2            (SC_START_FTHEAD1 + SC_LEN_FT_OBEX_HEADER)  // 10
#define SC_START_FTBODY             (SC_START_FTHEAD2 + SC_LEN_FT_OBEX_HEADER)  // 13

///*** Maths
#define HiByte(_z_)               ((_z_ >> 8) & 0xFF)
#define LoByte(_z_)               (_z_ & 0xFF)
#define BYTE_SWAP(_w_)            ((LoByte(_w_) << 8) | HiByte(_w_))

#define NTOH(_b0_, _b1_)               ((_b0_ << 8) | _b1_)
#define NTOHL(_b0_, _b1_, _b2_, _b3_)  ((_b0_ << 24) | (_b1_ << 16) | (_b2_ << 8) | (_b3_))


// ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// VARIABLES
// ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining VARIABLE  var_SC_BYTE
// 
VARIABLE var_SC_BYTE
{
    LABEL		"SC Data byte";
    CLASS		LOCAL;
    TYPE		UNSIGNED_INTEGER(1)
	{
		DEFAULT_VALUE		0;
		EDIT_FORMAT		"02X";
		DISPLAY_FORMAT		"02X";
	}
	WRITE_TIMEOUT	SC_TIMEOUT;
	READ_TIMEOUT	SC_TIMEOUT;
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining VARIABLE  var_SC_REQ_Buffer_SIZE
// 
VARIABLE var_SC_REQ_Buffer_SIZE
{
    LABEL		"SC REQ buffer size";
    CLASS		LOCAL;
    TYPE		UNSIGNED_INTEGER(2)
	{
		DEFAULT_VALUE	SC_TELE_LEN_MAX;
	}
}
//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining LIST  var_SC_REQ_Buffer
//   - Send buffer for SC transfers (with variable length)
LIST var_SC_REQ_Buffer
{
	LABEL		"SC Request buffer";
	TYPE		var_SC_BYTE;
	CAPACITY	SC_TELE_LEN_MAX;
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining VARIABLE  var_SC_RSP_Buffer_SIZE
// 
VARIABLE var_SC_RSP_Buffer_SIZE
{
    LABEL		"SC RSP buffer size";
    CLASS		LOCAL;
    TYPE		UNSIGNED_INTEGER(2)
	{
		DEFAULT_VALUE	SC_TELE_LEN_MAX;
	}
}
//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining LIST  var_SC_RSP_Buffer
//   - Receive buffer for SC transfers (with variable length)
LIST var_SC_RSP_Buffer
{
	LABEL		"SC Response buffer";
	TYPE		var_SC_BYTE;
	CAPACITY	SC_TELE_LEN_MAX;
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining single Buffer VARIABLEs
// 
var_SC_Byte000 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte001 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte002 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte003 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte004 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte005 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte006 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte007 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte008 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte009 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte010 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte011 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte012 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte013 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte014 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte015 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte016 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte017 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte018 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte019 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte020 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte021 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte022 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte023 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte024 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte025 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte026 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte027 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte028 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte029 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte030 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte031 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte032 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte033 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte034 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte035 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte036 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte037 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte038 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte039 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte040 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte041 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte042 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte043 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte044 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte045 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte046 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte047 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte048 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte049 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte050 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte051 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte052 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte053 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte054 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte055 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte056 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte057 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte058 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte059 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte060 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte061 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte062 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte063 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte064 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte065 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte066 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte067 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte068 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte069 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte070 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte071 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte072 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte073 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte074 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte075 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte076 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte077 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte078 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte079 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte080 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte081 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte082 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte083 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte084 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte085 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte086 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte087 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte088 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte089 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte090 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte091 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte092 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte093 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte094 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte095 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte096 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte097 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte098 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte099 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte100 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte101 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte102 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte103 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte104 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte105 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte106 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte107 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte108 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte109 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte110 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte111 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte112 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte113 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte114 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte115 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte116 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte117 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte118 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte119 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte120 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte121 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte122 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte123 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte124 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte125 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte126 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte127 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte128 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte129 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte130 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte131 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte132 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte133 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte134 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte135 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte136 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte137 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte138 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte139 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte140 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte141 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte142 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte143 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte144 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte145 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte146 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte147 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte148 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte149 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte150 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte151 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte152 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte153 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte154 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte155 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte156 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte157 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte158 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte159 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte160 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte161 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte162 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte163 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte164 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte165 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte166 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte167 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte168 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte169 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte170 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte171 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte172 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte173 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte174 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte175 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte176 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte177 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte178 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte179 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte180 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte181 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte182 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte183 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte184 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte185 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte186 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte187 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte188 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte189 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte190 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte191 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte192 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte193 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte194 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte195 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte196 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte197 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte198 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte199 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte200 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte201 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte202 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte203 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte204 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte205 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte206 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte207 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte208 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte209 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte210 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte211 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte212 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte213 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte214 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte215 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte216 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte217 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte218 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte219 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte220 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte221 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte222 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte223 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte224 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte225 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte226 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte227 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte228 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte229 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte230 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte231 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte232 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte233 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte234 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte235 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte236 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte237 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte238 LIKE VARIABLE var_SC_BYTE{}
var_SC_Byte239 LIKE VARIABLE var_SC_BYTE{}

// ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// COMMANDS
// ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining COMMAND write_SC_Request
//   - write service channel request to client (variable telegram length)
COMMAND write_SC_Request
{
	SLOT		DPV1_ACCESS_SLOT;
	INDEX		DPV1_FT_REQ_INDEX;
	OPERATION WRITE;
	TRANSACTION
	{
		REQUEST
		{
			var_SC_REQ_Buffer
		}
		REPLY
		{
		}
	}
}
//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining COMMAND read_SC_Response
//   - read service channel response from client (variable telegram length)
COMMAND read_SC_Response
{
	SLOT		DPV1_ACCESS_SLOT;
	INDEX		DPV1_FT_RSP_INDEX;
	OPERATION READ;
	TRANSACTION
	{
		REQUEST
		{
		}
		REPLY
		{
			var_SC_RSP_Buffer
		}
	}
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining COMMAND write_SC_Request009
//   - write service channel request to client (telegram length 9 bytes)
COMMAND write_SC_Request009
{
	SLOT		DPV1_ACCESS_SLOT;
	INDEX		DPV1_FT_REQ_INDEX;
	OPERATION WRITE;
	TRANSACTION
	{
		REQUEST
		{
			var_SC_Byte000,
			var_SC_Byte001,
			var_SC_Byte002,
			var_SC_Byte003,
			var_SC_Byte004,
			var_SC_Byte005,
			var_SC_Byte006,
			var_SC_Byte007,
			var_SC_Byte008
		}
		REPLY
		{
		}
	}
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining COMMAND write_SC_Request011
//   - write service channel request to client (telegram length 11 bytes)
COMMAND write_SC_Request011
{
	SLOT		DPV1_ACCESS_SLOT;
	INDEX		DPV1_FT_REQ_INDEX;
	OPERATION WRITE;
	TRANSACTION
	{
		REQUEST
		{
			var_SC_Byte000,
			var_SC_Byte001,
			var_SC_Byte002,
			var_SC_Byte003,
			var_SC_Byte004,
			var_SC_Byte005,
			var_SC_Byte006,
			var_SC_Byte007,
			var_SC_Byte008,
			var_SC_Byte009,
			var_SC_Byte010
		}
		REPLY
		{
		}
	}
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining COMMAND write_SC_Request012
//   - write service channel request to client (telegram length 12 bytes)
COMMAND write_SC_Request012
{
	SLOT		DPV1_ACCESS_SLOT;
	INDEX		DPV1_FT_REQ_INDEX;
	OPERATION WRITE;
	TRANSACTION
	{
		REQUEST
		{
			var_SC_Byte000,
			var_SC_Byte001,
			var_SC_Byte002,
			var_SC_Byte003,
			var_SC_Byte004,
			var_SC_Byte005,
			var_SC_Byte006,
			var_SC_Byte007,
			var_SC_Byte008,
			var_SC_Byte009,
			var_SC_Byte010,
			var_SC_Byte011
		}
		REPLY
		{
		}
	}
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining COMMAND write_SC_Request014
//   - write service channel request to client (telegram length 14 bytes)
COMMAND write_SC_Request014
{
	SLOT		DPV1_ACCESS_SLOT;
	INDEX		DPV1_FT_REQ_INDEX;
	OPERATION WRITE;
	TRANSACTION
	{
		REQUEST
		{
			var_SC_Byte000,
			var_SC_Byte001,
			var_SC_Byte002,
			var_SC_Byte003,
			var_SC_Byte004,
			var_SC_Byte005,
			var_SC_Byte006,
			var_SC_Byte007,
			var_SC_Byte008,
			var_SC_Byte009,
			var_SC_Byte010,
			var_SC_Byte011,
			var_SC_Byte012,
			var_SC_Byte013
		}
		REPLY
		{
		}
	}
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining COMMAND write_SC_Request019
//   - write service channel request to client (telegram length 19 bytes)
COMMAND write_SC_Request019
{
	SLOT		DPV1_ACCESS_SLOT;
	INDEX		DPV1_FT_REQ_INDEX;
	OPERATION WRITE;
	TRANSACTION
	{
		REQUEST
		{
			var_SC_Byte000,
			var_SC_Byte001,
			var_SC_Byte002,
			var_SC_Byte003,
			var_SC_Byte004,
			var_SC_Byte005,
			var_SC_Byte006,
			var_SC_Byte007,
			var_SC_Byte008,
			var_SC_Byte009,
			var_SC_Byte010,
			var_SC_Byte011,
			var_SC_Byte012,
			var_SC_Byte013,
			var_SC_Byte014,
			var_SC_Byte015,
			var_SC_Byte016, 
			var_SC_Byte017, 
			var_SC_Byte018  
		}
		REPLY
		{
		}
	}
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining COMMAND write_SC_Request035
//   - write service channel request to client (telegram length 35 bytes)
COMMAND write_SC_Request035
{
	SLOT		DPV1_ACCESS_SLOT;
	INDEX		DPV1_FT_REQ_INDEX;
	OPERATION WRITE;
	TRANSACTION
	{
		REQUEST
		{
			var_SC_Byte000,
			var_SC_Byte001,
			var_SC_Byte002,
			var_SC_Byte003,
			var_SC_Byte004,
			var_SC_Byte005,
			var_SC_Byte006,
			var_SC_Byte007,
			var_SC_Byte008,
			var_SC_Byte009,
			var_SC_Byte010,
			var_SC_Byte011,
			var_SC_Byte012,
			var_SC_Byte013,
			var_SC_Byte014,
			var_SC_Byte015,
			var_SC_Byte016,
			var_SC_Byte017,
			var_SC_Byte018,
			var_SC_Byte019,
			var_SC_Byte020,
			var_SC_Byte021,
			var_SC_Byte022,
			var_SC_Byte023,
			var_SC_Byte024,
			var_SC_Byte025,
			var_SC_Byte026,
			var_SC_Byte027,
			var_SC_Byte028,
			var_SC_Byte029,
			var_SC_Byte030,
			var_SC_Byte031,
			var_SC_Byte032,
			var_SC_Byte033,
			var_SC_Byte034
		}
		REPLY
		{
		}
	}
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining COMMAND write_SC_Request055
//   - write service channel request to client (telegram length 55 bytes)
COMMAND write_SC_Request055
{
	SLOT		DPV1_ACCESS_SLOT;
	INDEX		DPV1_FT_REQ_INDEX;
	OPERATION WRITE;
	TRANSACTION
	{
		REQUEST
		{
			var_SC_Byte000,
			var_SC_Byte001,
			var_SC_Byte002,
			var_SC_Byte003,
			var_SC_Byte004,
			var_SC_Byte005,
			var_SC_Byte006,
			var_SC_Byte007,
			var_SC_Byte008,
			var_SC_Byte009,
			var_SC_Byte010,
			var_SC_Byte011,
			var_SC_Byte012,
			var_SC_Byte013,
			var_SC_Byte014,
			var_SC_Byte015,
			var_SC_Byte016,
			var_SC_Byte017,
			var_SC_Byte018,
			var_SC_Byte019,
			var_SC_Byte020,
			var_SC_Byte021,
			var_SC_Byte022,
			var_SC_Byte023,
			var_SC_Byte024,
			var_SC_Byte025,
			var_SC_Byte026,
			var_SC_Byte027,
			var_SC_Byte028,
			var_SC_Byte029,
			var_SC_Byte030,
			var_SC_Byte031,
			var_SC_Byte032,
			var_SC_Byte033,
			var_SC_Byte034,
			var_SC_Byte035,
			var_SC_Byte036,
			var_SC_Byte037,
			var_SC_Byte038,
			var_SC_Byte039,
			var_SC_Byte040,
			var_SC_Byte041,
			var_SC_Byte042,
			var_SC_Byte043,
			var_SC_Byte044,
			var_SC_Byte045,
			var_SC_Byte046,
			var_SC_Byte047,
			var_SC_Byte048,
			var_SC_Byte049,
			var_SC_Byte050,
			var_SC_Byte051,
			var_SC_Byte052,
			var_SC_Byte053,
			var_SC_Byte054
		}
		REPLY
		{
		}
	}
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining COMMAND write_SC_Request061
//   - write service channel request to client (telegram length 61 bytes)
COMMAND write_SC_Request061
{
	SLOT		DPV1_ACCESS_SLOT;
	INDEX		DPV1_FT_REQ_INDEX;
	OPERATION WRITE;
	TRANSACTION
	{
		REQUEST
		{
			var_SC_Byte000,
			var_SC_Byte001,
			var_SC_Byte002,
			var_SC_Byte003,
			var_SC_Byte004,
			var_SC_Byte005,
			var_SC_Byte006,
			var_SC_Byte007,
			var_SC_Byte008,
			var_SC_Byte009,
			var_SC_Byte010,
			var_SC_Byte011,
			var_SC_Byte012,
			var_SC_Byte013,
			var_SC_Byte014,
			var_SC_Byte015,
			var_SC_Byte016,
			var_SC_Byte017,
			var_SC_Byte018,
			var_SC_Byte019,
			var_SC_Byte020,
			var_SC_Byte021,
			var_SC_Byte022,
			var_SC_Byte023,
			var_SC_Byte024,
			var_SC_Byte025,
			var_SC_Byte026,
			var_SC_Byte027,
			var_SC_Byte028,
			var_SC_Byte029,
			var_SC_Byte030,
			var_SC_Byte031,
			var_SC_Byte032,
			var_SC_Byte033,
			var_SC_Byte034,
			var_SC_Byte035,
			var_SC_Byte036,
			var_SC_Byte037,
			var_SC_Byte038,
			var_SC_Byte039,
			var_SC_Byte040,
			var_SC_Byte041,
			var_SC_Byte042,
			var_SC_Byte043,
			var_SC_Byte044,
			var_SC_Byte045,
			var_SC_Byte046,
			var_SC_Byte047,
			var_SC_Byte048,
			var_SC_Byte049,
			var_SC_Byte050,
			var_SC_Byte051,
			var_SC_Byte052,
			var_SC_Byte053,
			var_SC_Byte054,
			var_SC_Byte055,
			var_SC_Byte056,
			var_SC_Byte057,
			var_SC_Byte058,
			var_SC_Byte059,
			var_SC_Byte060
		}
		REPLY
		{
		}
	}
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining COMMAND read_SC_Response240
//   - read service channel response from client (telegram length 240 bytes)
COMMAND read_SC_Response240
{
	SLOT		DPV1_ACCESS_SLOT;
	INDEX		DPV1_FT_RSP_INDEX;
	OPERATION READ;
	TRANSACTION
	{
		REQUEST
		{
		}
		REPLY
		{
			var_SC_Byte000,
			var_SC_Byte001,
			var_SC_Byte002,
			var_SC_Byte003,
			var_SC_Byte004,
			var_SC_Byte005,
			var_SC_Byte006,
			var_SC_Byte007,
			var_SC_Byte008,
			var_SC_Byte009,
			var_SC_Byte010,
			var_SC_Byte011,
			var_SC_Byte012,
			var_SC_Byte013,
			var_SC_Byte014,
			var_SC_Byte015,
			var_SC_Byte016,
			var_SC_Byte017,
			var_SC_Byte018,
			var_SC_Byte019,
			var_SC_Byte020,
			var_SC_Byte021,
			var_SC_Byte022,
			var_SC_Byte023,
			var_SC_Byte024,
			var_SC_Byte025,
			var_SC_Byte026,
			var_SC_Byte027,
			var_SC_Byte028,
			var_SC_Byte029,
			var_SC_Byte030,
			var_SC_Byte031,
			var_SC_Byte032,
			var_SC_Byte033,
			var_SC_Byte034,
			var_SC_Byte035,
			var_SC_Byte036,
			var_SC_Byte037,
			var_SC_Byte038,
			var_SC_Byte039,
			var_SC_Byte040,
			var_SC_Byte041,
			var_SC_Byte042,
			var_SC_Byte043,
			var_SC_Byte044,
			var_SC_Byte045,
			var_SC_Byte046,
			var_SC_Byte047,
			var_SC_Byte048,
			var_SC_Byte049,
			var_SC_Byte050,
			var_SC_Byte051,
			var_SC_Byte052,
			var_SC_Byte053,
			var_SC_Byte054,
			var_SC_Byte055,
			var_SC_Byte056,
			var_SC_Byte057,
			var_SC_Byte058,
			var_SC_Byte059,
			var_SC_Byte060,
			var_SC_Byte061,
			var_SC_Byte062,
			var_SC_Byte063,
			var_SC_Byte064,
			var_SC_Byte065,
			var_SC_Byte066,
			var_SC_Byte067,
			var_SC_Byte068,
			var_SC_Byte069,
			var_SC_Byte070,
			var_SC_Byte071,
			var_SC_Byte072,
			var_SC_Byte073,
			var_SC_Byte074,
			var_SC_Byte075,
			var_SC_Byte076,
			var_SC_Byte077,
			var_SC_Byte078,
			var_SC_Byte079,
			var_SC_Byte080,
			var_SC_Byte081,
			var_SC_Byte082,
			var_SC_Byte083,
			var_SC_Byte084,
			var_SC_Byte085,
			var_SC_Byte086,
			var_SC_Byte087,
			var_SC_Byte088,
			var_SC_Byte089,
			var_SC_Byte090,
			var_SC_Byte091,
			var_SC_Byte092,
			var_SC_Byte093,
			var_SC_Byte094,
			var_SC_Byte095,
			var_SC_Byte096,
			var_SC_Byte097,
			var_SC_Byte098,
			var_SC_Byte099,
			var_SC_Byte100,
			var_SC_Byte101,
			var_SC_Byte102,
			var_SC_Byte103,
			var_SC_Byte104,
			var_SC_Byte105,
			var_SC_Byte106,
			var_SC_Byte107,
			var_SC_Byte108,
			var_SC_Byte109,
			var_SC_Byte110,
			var_SC_Byte111,
			var_SC_Byte112,
			var_SC_Byte113,
			var_SC_Byte114,
			var_SC_Byte115,
			var_SC_Byte116,
			var_SC_Byte117,
			var_SC_Byte118,
			var_SC_Byte119,
			var_SC_Byte120,
			var_SC_Byte121,
			var_SC_Byte122,
			var_SC_Byte123,
			var_SC_Byte124,
			var_SC_Byte125,
			var_SC_Byte126,
			var_SC_Byte127,
			var_SC_Byte128,
			var_SC_Byte129,
			var_SC_Byte130,
			var_SC_Byte131,
			var_SC_Byte132,
			var_SC_Byte133,
			var_SC_Byte134,
			var_SC_Byte135,
			var_SC_Byte136,
			var_SC_Byte137,
			var_SC_Byte138,
			var_SC_Byte139,
			var_SC_Byte140,
			var_SC_Byte141,
			var_SC_Byte142,
			var_SC_Byte143,
			var_SC_Byte144,
			var_SC_Byte145,
			var_SC_Byte146,
			var_SC_Byte147,
			var_SC_Byte148,
			var_SC_Byte149,
			var_SC_Byte150,
			var_SC_Byte151,
			var_SC_Byte152,
			var_SC_Byte153,
			var_SC_Byte154,
			var_SC_Byte155,
			var_SC_Byte156,
			var_SC_Byte157,
			var_SC_Byte158,
			var_SC_Byte159,
			var_SC_Byte160,
			var_SC_Byte161,
			var_SC_Byte162,
			var_SC_Byte163,
			var_SC_Byte164,
			var_SC_Byte165,
			var_SC_Byte166,
			var_SC_Byte167,
			var_SC_Byte168,
			var_SC_Byte169,
			var_SC_Byte170,
			var_SC_Byte171,
			var_SC_Byte172,
			var_SC_Byte173,
			var_SC_Byte174,
			var_SC_Byte175,
			var_SC_Byte176,
			var_SC_Byte177,
			var_SC_Byte178,
			var_SC_Byte179,
			var_SC_Byte180,
			var_SC_Byte181,
			var_SC_Byte182,
			var_SC_Byte183,
			var_SC_Byte184,
			var_SC_Byte185,
			var_SC_Byte186,
			var_SC_Byte187,
			var_SC_Byte188,
			var_SC_Byte189,
			var_SC_Byte190,
			var_SC_Byte191,
			var_SC_Byte192,
			var_SC_Byte193,
			var_SC_Byte194,
			var_SC_Byte195,
			var_SC_Byte196,
			var_SC_Byte197,
			var_SC_Byte198,
			var_SC_Byte199,
			var_SC_Byte200,
			var_SC_Byte201,
			var_SC_Byte202,
			var_SC_Byte203,
			var_SC_Byte204,
			var_SC_Byte205,
			var_SC_Byte206,
			var_SC_Byte207,
			var_SC_Byte208,
			var_SC_Byte209,
			var_SC_Byte210,
			var_SC_Byte211,
			var_SC_Byte212,
			var_SC_Byte213,
			var_SC_Byte214,
			var_SC_Byte215,
			var_SC_Byte216,
			var_SC_Byte217,
			var_SC_Byte218,
			var_SC_Byte219,
			var_SC_Byte220,
			var_SC_Byte221,
			var_SC_Byte222,
			var_SC_Byte223,
			var_SC_Byte224,
			var_SC_Byte225,
			var_SC_Byte226,
			var_SC_Byte227,
			var_SC_Byte228,
			var_SC_Byte229,
			var_SC_Byte230,
			var_SC_Byte231,
			var_SC_Byte232,
			var_SC_Byte233,
			var_SC_Byte234,
			var_SC_Byte235,
			var_SC_Byte236,
			var_SC_Byte237,
			var_SC_Byte238,
			var_SC_Byte239
		}
	}
}


// ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// METHODS
// ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#ifndef SCCOM_WITH_LISTS
//---------------------------------------------------------------------------------------------------------------------
//~
//   METHOD send_SC_Request
// 
METHOD send_SC_Request
{
	LABEL		"send_SC_Request";
	TYPE    unsigned int;
	DEFINITION
	{
		unsigned int rc;
		unsigned int len;
		unsigned int i;
		unsigned int trials;

		// var_SC_REQ_Buffer kopieren und zur Länge korrespondierendes Kommando aufrufen
		len= var_SC_REQ_Buffer.COUNT;
		_TRACE("### --> send_SC_Request(): SC_HDR= " + itoa(var_SC_REQ_Buffer[0]) + " " + itoa(var_SC_REQ_Buffer[1]) + " " + itoa(var_SC_REQ_Buffer[2]) + " " + itoa(var_SC_REQ_Buffer[3]) + " " + itoa(var_SC_REQ_Buffer[4]) + ", len= " + itoa(len));
		rc= SC_ERROR;
		switch (len)
		{
			default:
			{
			    PUT_MESSAGE("send_SC_Request: buffer[" + itoa(len) + "] not supported yet!");
				return (SC_ERROR);
				break;
			}
			case 9:
			{
				i= 0;
				var_SC_Byte000= var_SC_REQ_Buffer[i++];
				var_SC_Byte001= var_SC_REQ_Buffer[i++];
				var_SC_Byte002= var_SC_REQ_Buffer[i++];
				var_SC_Byte003= var_SC_REQ_Buffer[i++];
				var_SC_Byte004= var_SC_REQ_Buffer[i++];
				var_SC_Byte005= var_SC_REQ_Buffer[i++];
				var_SC_Byte006= var_SC_REQ_Buffer[i++];
				var_SC_Byte007= var_SC_REQ_Buffer[i++];
				var_SC_Byte008= var_SC_REQ_Buffer[i++];

				trials= 0;
				do 
				{
					rc= WriteCommand(write_SC_Request009);
					trials++;
				} while (trials < 3 && rc != 0);
				break;
			}
			case 11:
			{
				i= 0;
				var_SC_Byte000= var_SC_REQ_Buffer[i++];
				var_SC_Byte001= var_SC_REQ_Buffer[i++];
				var_SC_Byte002= var_SC_REQ_Buffer[i++];
				var_SC_Byte003= var_SC_REQ_Buffer[i++];
				var_SC_Byte004= var_SC_REQ_Buffer[i++];
				var_SC_Byte005= var_SC_REQ_Buffer[i++];
				var_SC_Byte006= var_SC_REQ_Buffer[i++];
				var_SC_Byte007= var_SC_REQ_Buffer[i++];
				var_SC_Byte008= var_SC_REQ_Buffer[i++];
				var_SC_Byte009= var_SC_REQ_Buffer[i++];
				var_SC_Byte010= var_SC_REQ_Buffer[i++];

				trials= 0;
				do 
				{
					rc= WriteCommand(write_SC_Request011);
					trials++;
				} while (trials < 3 && rc != 0);
				break;
			}
			case 12:
			{
				i= 0;
				var_SC_Byte000= var_SC_REQ_Buffer[i++];
				var_SC_Byte001= var_SC_REQ_Buffer[i++];
				var_SC_Byte002= var_SC_REQ_Buffer[i++];
				var_SC_Byte003= var_SC_REQ_Buffer[i++];
				var_SC_Byte004= var_SC_REQ_Buffer[i++];
				var_SC_Byte005= var_SC_REQ_Buffer[i++];
				var_SC_Byte006= var_SC_REQ_Buffer[i++];
				var_SC_Byte007= var_SC_REQ_Buffer[i++];
				var_SC_Byte008= var_SC_REQ_Buffer[i++];
				var_SC_Byte009= var_SC_REQ_Buffer[i++];
				var_SC_Byte010= var_SC_REQ_Buffer[i++];
				var_SC_Byte011= var_SC_REQ_Buffer[i++];

				trials= 0;
				do 
				{
					rc= WriteCommand(write_SC_Request012);
					trials++;
				} while (trials < 3 && rc != 0);
				break;
			}
			case 14:
			{
				i= 0;
				var_SC_Byte000= var_SC_REQ_Buffer[i++];
				var_SC_Byte001= var_SC_REQ_Buffer[i++];
				var_SC_Byte002= var_SC_REQ_Buffer[i++];
				var_SC_Byte003= var_SC_REQ_Buffer[i++];
				var_SC_Byte004= var_SC_REQ_Buffer[i++];
				var_SC_Byte005= var_SC_REQ_Buffer[i++];
				var_SC_Byte006= var_SC_REQ_Buffer[i++];
				var_SC_Byte007= var_SC_REQ_Buffer[i++];
				var_SC_Byte008= var_SC_REQ_Buffer[i++];
				var_SC_Byte009= var_SC_REQ_Buffer[i++];
				var_SC_Byte010= var_SC_REQ_Buffer[i++];
				var_SC_Byte011= var_SC_REQ_Buffer[i++];
				var_SC_Byte012= var_SC_REQ_Buffer[i++];
				var_SC_Byte013= var_SC_REQ_Buffer[i++];

				trials= 0;
				do 
				{
					rc= WriteCommand(write_SC_Request014);
					trials++;
				} while (trials < 3 && rc != 0);
				break;
			}
			case 19:
			{
				i= 0;
				var_SC_Byte000= var_SC_REQ_Buffer[i++];
				var_SC_Byte001= var_SC_REQ_Buffer[i++];
				var_SC_Byte002= var_SC_REQ_Buffer[i++];
				var_SC_Byte003= var_SC_REQ_Buffer[i++];
				var_SC_Byte004= var_SC_REQ_Buffer[i++];
				var_SC_Byte005= var_SC_REQ_Buffer[i++];
				var_SC_Byte006= var_SC_REQ_Buffer[i++];
				var_SC_Byte007= var_SC_REQ_Buffer[i++];
				var_SC_Byte008= var_SC_REQ_Buffer[i++];
				var_SC_Byte009= var_SC_REQ_Buffer[i++];
				var_SC_Byte010= var_SC_REQ_Buffer[i++];
				var_SC_Byte011= var_SC_REQ_Buffer[i++];
				var_SC_Byte012= var_SC_REQ_Buffer[i++];
				var_SC_Byte013= var_SC_REQ_Buffer[i++];
				var_SC_Byte014= var_SC_REQ_Buffer[i++];
				var_SC_Byte015= var_SC_REQ_Buffer[i++];
				var_SC_Byte016= var_SC_REQ_Buffer[i++];
				var_SC_Byte017= var_SC_REQ_Buffer[i++];
				var_SC_Byte018= var_SC_REQ_Buffer[i++];

				trials= 0;
				do 
				{
					rc= WriteCommand(write_SC_Request019);
					trials++;
				} while (trials < 3 && rc != 0);
				break;
			}
			case 35:
			{
				i= 0;
				var_SC_Byte000= var_SC_REQ_Buffer[i++];
				var_SC_Byte001= var_SC_REQ_Buffer[i++];
				var_SC_Byte002= var_SC_REQ_Buffer[i++];
				var_SC_Byte003= var_SC_REQ_Buffer[i++];
				var_SC_Byte004= var_SC_REQ_Buffer[i++];
				var_SC_Byte005= var_SC_REQ_Buffer[i++];
				var_SC_Byte006= var_SC_REQ_Buffer[i++];
				var_SC_Byte007= var_SC_REQ_Buffer[i++];
				var_SC_Byte008= var_SC_REQ_Buffer[i++];
				var_SC_Byte009= var_SC_REQ_Buffer[i++];
				var_SC_Byte010= var_SC_REQ_Buffer[i++];
				var_SC_Byte011= var_SC_REQ_Buffer[i++];
				var_SC_Byte012= var_SC_REQ_Buffer[i++];
				var_SC_Byte013= var_SC_REQ_Buffer[i++];
				var_SC_Byte014= var_SC_REQ_Buffer[i++];
				var_SC_Byte015= var_SC_REQ_Buffer[i++];
				var_SC_Byte016= var_SC_REQ_Buffer[i++];
				var_SC_Byte017= var_SC_REQ_Buffer[i++];
				var_SC_Byte018= var_SC_REQ_Buffer[i++];
				var_SC_Byte019= var_SC_REQ_Buffer[i++];
				var_SC_Byte020= var_SC_REQ_Buffer[i++];
				var_SC_Byte021= var_SC_REQ_Buffer[i++];
				var_SC_Byte022= var_SC_REQ_Buffer[i++];
				var_SC_Byte023= var_SC_REQ_Buffer[i++];
				var_SC_Byte024= var_SC_REQ_Buffer[i++];
				var_SC_Byte025= var_SC_REQ_Buffer[i++];
				var_SC_Byte026= var_SC_REQ_Buffer[i++];
				var_SC_Byte027= var_SC_REQ_Buffer[i++];
				var_SC_Byte028= var_SC_REQ_Buffer[i++];
				var_SC_Byte029= var_SC_REQ_Buffer[i++];
				var_SC_Byte030= var_SC_REQ_Buffer[i++];
				var_SC_Byte031= var_SC_REQ_Buffer[i++];
				var_SC_Byte032= var_SC_REQ_Buffer[i++];
				var_SC_Byte033= var_SC_REQ_Buffer[i++];
				var_SC_Byte034= var_SC_REQ_Buffer[i++];

				trials= 0;
				do 
				{
					rc= WriteCommand(write_SC_Request035);
					trials++;
				} while (trials < 3 && rc != 0);
				break;
			}
			case 55:
			{
				i= 0;
				var_SC_Byte000= var_SC_REQ_Buffer[i++];
				var_SC_Byte001= var_SC_REQ_Buffer[i++];
				var_SC_Byte002= var_SC_REQ_Buffer[i++];
				var_SC_Byte003= var_SC_REQ_Buffer[i++];
				var_SC_Byte004= var_SC_REQ_Buffer[i++];
				var_SC_Byte005= var_SC_REQ_Buffer[i++];
				var_SC_Byte006= var_SC_REQ_Buffer[i++];
				var_SC_Byte007= var_SC_REQ_Buffer[i++];
				var_SC_Byte008= var_SC_REQ_Buffer[i++];
				var_SC_Byte009= var_SC_REQ_Buffer[i++];
				var_SC_Byte010= var_SC_REQ_Buffer[i++];
				var_SC_Byte011= var_SC_REQ_Buffer[i++];
				var_SC_Byte012= var_SC_REQ_Buffer[i++];
				var_SC_Byte013= var_SC_REQ_Buffer[i++];
				var_SC_Byte014= var_SC_REQ_Buffer[i++];
				var_SC_Byte015= var_SC_REQ_Buffer[i++];
				var_SC_Byte016= var_SC_REQ_Buffer[i++];
				var_SC_Byte017= var_SC_REQ_Buffer[i++];
				var_SC_Byte018= var_SC_REQ_Buffer[i++];
				var_SC_Byte019= var_SC_REQ_Buffer[i++];
				var_SC_Byte020= var_SC_REQ_Buffer[i++];
				var_SC_Byte021= var_SC_REQ_Buffer[i++];
				var_SC_Byte022= var_SC_REQ_Buffer[i++];
				var_SC_Byte023= var_SC_REQ_Buffer[i++];
				var_SC_Byte024= var_SC_REQ_Buffer[i++];
				var_SC_Byte025= var_SC_REQ_Buffer[i++];
				var_SC_Byte026= var_SC_REQ_Buffer[i++];
				var_SC_Byte027= var_SC_REQ_Buffer[i++];
				var_SC_Byte028= var_SC_REQ_Buffer[i++];
				var_SC_Byte029= var_SC_REQ_Buffer[i++];
				var_SC_Byte030= var_SC_REQ_Buffer[i++];
				var_SC_Byte031= var_SC_REQ_Buffer[i++];
				var_SC_Byte032= var_SC_REQ_Buffer[i++];
				var_SC_Byte033= var_SC_REQ_Buffer[i++];
				var_SC_Byte034= var_SC_REQ_Buffer[i++];
				var_SC_Byte035= var_SC_REQ_Buffer[i++];
				var_SC_Byte036= var_SC_REQ_Buffer[i++];
				var_SC_Byte037= var_SC_REQ_Buffer[i++];
				var_SC_Byte038= var_SC_REQ_Buffer[i++];
				var_SC_Byte039= var_SC_REQ_Buffer[i++];
				var_SC_Byte040= var_SC_REQ_Buffer[i++];
				var_SC_Byte041= var_SC_REQ_Buffer[i++];
				var_SC_Byte042= var_SC_REQ_Buffer[i++];
				var_SC_Byte043= var_SC_REQ_Buffer[i++];
				var_SC_Byte044= var_SC_REQ_Buffer[i++];
				var_SC_Byte045= var_SC_REQ_Buffer[i++];
				var_SC_Byte046= var_SC_REQ_Buffer[i++];
				var_SC_Byte047= var_SC_REQ_Buffer[i++];
				var_SC_Byte048= var_SC_REQ_Buffer[i++];
				var_SC_Byte049= var_SC_REQ_Buffer[i++];
				var_SC_Byte050= var_SC_REQ_Buffer[i++];
				var_SC_Byte051= var_SC_REQ_Buffer[i++];
				var_SC_Byte052= var_SC_REQ_Buffer[i++];
				var_SC_Byte053= var_SC_REQ_Buffer[i++];
				var_SC_Byte054= var_SC_REQ_Buffer[i++];

				trials= 0;
				do 
				{
					rc= WriteCommand(write_SC_Request055);
					trials++;
				} while (trials < 3 && rc != 0);
				break;
			}
			case 61:
			{
				i= 0;
				var_SC_Byte000= var_SC_REQ_Buffer[i++];
				var_SC_Byte001= var_SC_REQ_Buffer[i++];
				var_SC_Byte002= var_SC_REQ_Buffer[i++];
				var_SC_Byte003= var_SC_REQ_Buffer[i++];
				var_SC_Byte004= var_SC_REQ_Buffer[i++];
				var_SC_Byte005= var_SC_REQ_Buffer[i++];
				var_SC_Byte006= var_SC_REQ_Buffer[i++];
				var_SC_Byte007= var_SC_REQ_Buffer[i++];
				var_SC_Byte008= var_SC_REQ_Buffer[i++];
				var_SC_Byte009= var_SC_REQ_Buffer[i++];
				var_SC_Byte010= var_SC_REQ_Buffer[i++];
				var_SC_Byte011= var_SC_REQ_Buffer[i++];
				var_SC_Byte012= var_SC_REQ_Buffer[i++];
				var_SC_Byte013= var_SC_REQ_Buffer[i++];
				var_SC_Byte014= var_SC_REQ_Buffer[i++];
				var_SC_Byte015= var_SC_REQ_Buffer[i++];
				var_SC_Byte016= var_SC_REQ_Buffer[i++];
				var_SC_Byte017= var_SC_REQ_Buffer[i++];
				var_SC_Byte018= var_SC_REQ_Buffer[i++];
				var_SC_Byte019= var_SC_REQ_Buffer[i++];
				var_SC_Byte020= var_SC_REQ_Buffer[i++];
				var_SC_Byte021= var_SC_REQ_Buffer[i++];
				var_SC_Byte022= var_SC_REQ_Buffer[i++];
				var_SC_Byte023= var_SC_REQ_Buffer[i++];
				var_SC_Byte024= var_SC_REQ_Buffer[i++];
				var_SC_Byte025= var_SC_REQ_Buffer[i++];
				var_SC_Byte026= var_SC_REQ_Buffer[i++];
				var_SC_Byte027= var_SC_REQ_Buffer[i++];
				var_SC_Byte028= var_SC_REQ_Buffer[i++];
				var_SC_Byte029= var_SC_REQ_Buffer[i++];
				var_SC_Byte030= var_SC_REQ_Buffer[i++];
				var_SC_Byte031= var_SC_REQ_Buffer[i++];
				var_SC_Byte032= var_SC_REQ_Buffer[i++];
				var_SC_Byte033= var_SC_REQ_Buffer[i++];
				var_SC_Byte034= var_SC_REQ_Buffer[i++];
				var_SC_Byte035= var_SC_REQ_Buffer[i++];
				var_SC_Byte036= var_SC_REQ_Buffer[i++];
				var_SC_Byte037= var_SC_REQ_Buffer[i++];
				var_SC_Byte038= var_SC_REQ_Buffer[i++];
				var_SC_Byte039= var_SC_REQ_Buffer[i++];
				var_SC_Byte040= var_SC_REQ_Buffer[i++];
				var_SC_Byte041= var_SC_REQ_Buffer[i++];
				var_SC_Byte042= var_SC_REQ_Buffer[i++];
				var_SC_Byte043= var_SC_REQ_Buffer[i++];
				var_SC_Byte044= var_SC_REQ_Buffer[i++];
				var_SC_Byte045= var_SC_REQ_Buffer[i++];
				var_SC_Byte046= var_SC_REQ_Buffer[i++];
				var_SC_Byte047= var_SC_REQ_Buffer[i++];
				var_SC_Byte048= var_SC_REQ_Buffer[i++];
				var_SC_Byte049= var_SC_REQ_Buffer[i++];
				var_SC_Byte050= var_SC_REQ_Buffer[i++];
				var_SC_Byte051= var_SC_REQ_Buffer[i++];
				var_SC_Byte052= var_SC_REQ_Buffer[i++];
				var_SC_Byte053= var_SC_REQ_Buffer[i++];
				var_SC_Byte054= var_SC_REQ_Buffer[i++];
				var_SC_Byte055= var_SC_REQ_Buffer[i++];
				var_SC_Byte056= var_SC_REQ_Buffer[i++];
				var_SC_Byte057= var_SC_REQ_Buffer[i++];
				var_SC_Byte058= var_SC_REQ_Buffer[i++];
				var_SC_Byte059= var_SC_REQ_Buffer[i++];
				var_SC_Byte060= var_SC_REQ_Buffer[i++];

				trials= 0;
				do 
				{
					rc= WriteCommand(write_SC_Request061);
					trials++;
				} while (trials < 3 && rc != 0);
				break;
			}
		}
		_TRACE("### --> send_SC_Request(): finished (rc= "+ itoa(rc) + ", trials= " + itoa(trials) + ")");
		if (rc == 0)
		{
			rc= SC_OKAY;
		}
		else
		{
			rc= SC_ERROR;
		}
		return (rc);
	}
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   METHOD receive_SC_Response
// 
METHOD receive_SC_Response
{
	LABEL		"receive_SC_Response";
	TYPE    unsigned int;
	DEFINITION
	{
		unsigned int rc;
		unsigned int i;
		unsigned int len;
		unsigned int trials;

		trials= 0;
		do 
		{
			rc= ReadCommand(read_SC_Response240);
			trials++;
		} while (trials < 3 && rc != 0);
		// wenn ReadCommand erfolgreich war
		if (0 == rc)
		{
			// Anzahl empfangene Bytes ermitteln
			if (var_SC_Byte000 == 0xFE)
			{
				// SC-HDR (erweitertes Format)
				len= (var_SC_Byte002 << 8) + var_SC_Byte001 + 7;
			}
			else
			{
				// SC-HDR (Standard-Format)
				len= var_SC_Byte002 + 7;
			}
			_TRACE("### --> receive_SC_Response(): SC_HDR= " + itoa(var_SC_Byte000) + " " + itoa(var_SC_Byte001) + " " + itoa(var_SC_Byte002) + " " + itoa(var_SC_Byte003) + " " + itoa(var_SC_Byte004) + ", total len= " + itoa(len));
			if (SC_TELE_LEN_MAX < len)
			{
				len= SC_TELE_LEN_MAX;
				_TRACE("### --> receive_SC_Response(): LENGTH descriptor exceeds maximum!!! (reduced to " + itoa(len) + ")");
			}
			// resize RSP buffer (5 octets header + pay load + 2 octets checksum)
			resize_SC_RSP_Buffer(len);
			// Empfangsbuffer einlesen
			i=0;
			// gültige Bytes in var_SC_RSP_Buffer kopieren (eigentlich "do .. while(i < len)")
			var_SC_RSP_Buffer[i++]= var_SC_Byte000; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte001; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte002; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte003; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte004; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte005; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte006; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte007; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte008; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte009; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte010; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte011; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte012; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte013; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte014; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte015; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte016; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte017; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte018; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte019; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte020; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte021; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte022; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte023; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte024; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte025; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte026; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte027; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte028; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte029; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte030; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte031; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte032; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte033; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte034; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte035; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte036; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte037; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte038; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte039; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte040; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte041; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte042; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte043; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte044; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte045; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte046; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte047; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte048; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte049; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte050; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte051; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte052; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte053; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte054; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte055; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte056; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte057; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte058; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte059; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte060; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte061; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte062; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte063; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte064; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte065; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte066; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte067; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte068; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte069; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte070; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte071; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte072; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte073; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte074; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte075; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte076; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte077; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte078; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte079; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte080; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte081; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte082; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte083; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte084; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte085; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte086; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte087; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte088; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte089; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte090; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte091; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte092; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte093; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte094; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte095; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte096; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte097; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte098; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte099; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte100; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte101; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte102; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte103; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte104; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte105; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte106; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte107; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte108; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte109; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte110; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte111; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte112; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte113; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte114; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte115; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte116; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte117; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte118; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte119; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte120; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte121; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte122; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte123; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte124; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte125; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte126; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte127; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte128; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte129; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte130; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte131; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte132; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte133; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte134; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte135; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte136; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte137; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte138; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte139; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte140; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte141; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte142; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte143; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte144; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte145; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte146; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte147; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte148; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte149; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte150; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte151; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte152; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte153; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte154; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte155; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte156; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte157; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte158; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte159; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte160; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte161; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte162; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte163; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte164; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte165; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte166; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte167; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte168; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte169; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte170; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte171; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte172; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte173; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte174; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte175; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte176; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte177; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte178; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte179; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte180; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte181; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte182; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte183; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte184; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte185; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte186; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte187; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte188; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte189; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte190; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte191; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte192; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte193; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte194; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte195; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte196; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte197; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte198; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte199; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte200; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte201; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte202; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte203; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte204; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte205; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte206; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte207; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte208; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte209; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte210; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte211; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte212; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte213; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte214; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte215; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte216; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte217; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte218; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte219; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte220; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte221; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte222; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte223; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte224; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte225; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte226; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte227; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte228; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte229; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte230; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte231; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte232; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte233; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte234; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte235; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte236; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte237; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte238; if (i >= len) return (SC_OKAY);
			var_SC_RSP_Buffer[i++]= var_SC_Byte239; if (i >= len) return (SC_OKAY);
			save_values();
		}
		if (rc == 0)
		{
			rc= SC_OKAY;
		}
		else
		{
			rc= SC_ERROR;
		}
		return (rc);
	}
}

#else
//---------------------------------------------------------------------------------------------------------------------
//~
//   METHOD send_SC_Request
// 
METHOD send_SC_Request
{
	LABEL		"send_SC_Request";
	TYPE    unsigned int;
	DEFINITION
	{
		unsigned int rc;
		unsigned int i;
		unsigned int trials;

		_TRACE("### --> send_SC_Request(): SC_HDR= " + itoa(var_SC_REQ_Buffer[0]) + " " + itoa(var_SC_REQ_Buffer[1]) + " " + itoa(var_SC_REQ_Buffer[2]) + " " + itoa(var_SC_REQ_Buffer[3]) + " " + itoa(var_SC_REQ_Buffer[4]) + ", len= " + itoa(var_SC_REQ_Buffer.COUNT));

		trials= 0;
		do 
		{
			rc= WriteCommand(write_SC_Request);
			trials++;
		} while (trials < 3 && rc != 0);
		_TRACE("### --> send_SC_Request(): finished (rc= "+ itoa(rc) + ", trials= " + itoa(trials) + ")");
		if (rc == 0)
		{
			rc= SC_OKAY;
		}
		else
		{
			rc= SC_ERROR;
		}
		return (rc);
	}
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   METHOD receive_SC_Response
// 
METHOD receive_SC_Response
{
	LABEL		"receive_SC_Response";
	TYPE    unsigned int;
	DEFINITION
	{
		unsigned int rc;
		unsigned int i;
		unsigned int len;
		unsigned int trials;

		trials= 0;
		do 
		{
			rc= ReadCommand(read_SC_Response);
			trials++;
		} while (trials < 3 && rc != 0);
		if (rc == 0)
		{
			_TRACE("### --> receive_SC_Response(): SC_HDR= " + itoa(var_SC_RSP_Buffer[0]) + " " + itoa(var_SC_RSP_Buffer[1]) + " " + itoa(var_SC_RSP_Buffer[2]) + " " + itoa(var_SC_RSP_Buffer[3]) + " " + itoa(var_SC_RSP_Buffer[4]) + ", len= " + itoa(var_SC_RSP_Buffer.COUNT));
			rc= SC_OKAY;
		}
		else
		{
			rc= SC_ERROR;
		}
		return (rc);
	}
}

#endif
//---------------------------------------------------------------------------------------------------------------------
//~
//   End of file
//---------------------------------------------------------------------------------------------------------------------
